/*
 * file CqDbSet.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqDbSet
 *
 * (C) Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */
package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.StpRepository;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;
import com.ibm.rational.wvcm.stpex.StpExEnumerationBase;


/**
 * A collection of user database resources. The user databases are known by
 * simple names within a CqDbSet, and thus the CqDbSet is fundamentally useful for
 * enumerating those database names.
 * <p>
 * The user-friendly specification for the location of a db-set has the form
 * <pre>
 *  <b>cq.dbset:</b><i>&lt;db-set-name&gt;
 * </pre>
 */
public interface CqDbSet
    extends CqDb, StpRepository
{
    /**
     * A list of the user databases to which the current user has access. The
     * list is the same as that returned by
     * getCurrentUser().getSubscribedDatabases(), but can be computed much more
     * efficiently.
     */
    PropertyName<ResourceList<CqUserDb>> ACCESSIBLE_DATABASES =
        new PropertyName<ResourceList<CqUserDb>>(PROPERTY_NAMESPACE, 
                                                 "accessible-databases");

    /**
     * Returns the value of the {@link #ACCESSIBLE_DATABASES} property as
     * defined by this proxy.
     * 
     * @return A list of the user databases that are in this DbSet. Will never
     *         be <b>null</b>, but may be empty.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #ACCESSIBLE_DATABASES} property.
     */
    ResourceList<CqUserDb> getAccessibleDatabases() throws WvcmException;
    
    /** A list of the user databases that are in this DbSet */
    PropertyName<ResourceList<CqUserDb>> USER_DATABASES =
        new PropertyName<ResourceList<CqUserDb>>(PROPERTY_NAMESPACE, "user-databases");

    /**
     * Returns the value of the {@link #USER_DATABASES} property as defined by
     * this proxy.
     * 
     * @return A list of the user databases that are in this DbSet. Will never
     *         be <b>null</b>, but may be empty.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #USER_DATABASES} property.
     */
    ResourceList<CqUserDb> getUserDatabases() throws WvcmException;
    
    /**
     * Indicates whether the current authentication algorithm is ClearQuest
     * first (then LDAP) or ClearQuest only (no LDAP).
     */
    PropertyName<AuthenticationAlgorithm> AUTHENTICATION_ALGORITHM =
        new PropertyName<AuthenticationAlgorithm>(PROPERTY_NAMESPACE,
                                                  "authentication-algorithm");

    /**
      * Returns the value of the {@link #AUTHENTICATION_ALGORITHM AUTHENTICATION_ALGORITHM}
      * property as defined by this proxy.
      * 
      * @return The AuthenticationAlgorithm value. Will never be <b>null</b>.
      * 
      * @throws WvcmException if this proxy does not define a value for the
      *             {@link #AUTHENTICATION_ALGORITHM AUTHENTICATION_ALGORITHM} property.
      */
     AuthenticationAlgorithm getAuthenticationAlgorithm() throws WvcmException;


    /**
     * Defines a new value for the
     * {@link #AUTHENTICATION_ALGORITHM AUTHENTICATION_ALGORITHM} property of
     * this proxy.
     * 
     * @param value An AuthenticationAlgorithm object specifying the new value
     *            of the property. Must not be <b>null</b>.
     */
     void setAuthenticationAlgorithm(AuthenticationAlgorithm value);
     
    /**
     * AuthenticationAlgorithm constants specify which authentication search
     * strategy is selected when a Rational ClearQuest user logs on.
     */
     enum AuthenticationAlgorithm implements StpExEnumeration
     {
        /**
         * Authenticate using traditional Rational ClearQuest user
         * authentication as the preference, and failing that, attempt to
         * authenticate using LDAP authentication.
         */
         CLEAR_QUEST_FIRST,
         
        /**
         * Traditional Rational ClearQuest user authentication. Does not
         * allow LDAP authentication. This is the default algorithm.
         */
         CLEAR_QUEST_ONLY;
     }

     /**
         * Answers whether or not this database set, using the CqProvider of
         * this proxy, provides user and group administration functions such as
         * the creation of new users and groups and the modification of user and
         * group properties.
         */
     static PropertyName<Boolean> IS_USER_ADMIN_ENABLED =
         new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-user-admin-enabled");

     /**
         * Returns the value of the
         * {@link #IS_USER_ADMIN_ENABLED IS_USER_ADMIN_ENABLED} property as
         * defined by this proxy.
         * 
         * @return <b>true</b> if user and group administration is supported by
         *         this database through this proxy's CqProvider; <b>false</b>
         *         otherwise.
         * 
         * @throws WvcmException
         *             if this proxy does not define a value for the
         *             {@link #IS_USER_ADMIN_ENABLED IS_USER_ADMIN_ENABLED}
         *             property.
         */
     boolean getIsUserAdminEnabled() throws WvcmException;

}
